#pragma once

#include <objmodel.h>

class CSysMonWnd : public Gtk::Window, public ISysMonWnd
{
public:
  CSysMonWnd();
  virtual ~CSysMonWnd();
  // ISysMonWnd interface
  bool_t addGraphWnd(void);
  const DeviceInfo& getDeviceInfo() const { return m_device; }
  IDeviceInfoWnd* getDeviceInfoWnd() const { return m_pDeviceInfoWnd; }
  window_handle_t getNativeWindow() { return *this; }
  IGraphWnd* getGraphWnd(unsigned int index) const {
    ASSERT(index < m_numGraphWnd);
    return m_ppGraphWnd[index];
  }
  unsigned int getNumGraphWnd() const { return m_numGraphWnd; }
  ISensorInfoWnd* getSensorInfoWnd() const { return m_pSensorInfoWnd; }
  void idle();
  bool_t removeGraphWnd(unsigned int index);
  bool_t removeGraphWnd(IGraphWnd* pGraphWnd);

protected:
  // Following members define main window default size, in pixels
  static const int m_defaultWidth = 640;
  static const int m_defaultHeight = 480;
  static const int m_minWidth = 520;
  static const int m_minHeight = 360;

  // Sensor update period, in milliseconds
  static const int m_sensorUpdatePeriod = 1000;

  static const int m_updateTimerNumber = 0;

  enum TabIndex {
    TabDevoceInfo = 0,
    TabSensorInfo = 1,
    TabSensorReadings = 2
  };
  unsigned int m_deviceIndex;
  unsigned int m_numGraphWnd;
  int m_updatePeriodIndex;
  TabIndex m_currentTab;
  DeviceInfo m_device;
  IGraphWnd** m_ppGraphWnd;
  IDeviceInfoWnd* m_pDeviceInfoWnd;
  ISensorInfoWnd* m_pSensorInfoWnd;

protected:
  Glib::RefPtr<Gdk::Pixbuf> m_iconPixbuf;
  Glib::RefPtr<Gtk::StatusIcon> m_trayIcon;
  Gtk::Button m_aboutCtl;
  Gtk::Menu m_trayMenu;
  Gtk::MenuItem m_trayMenuRestore;
  Gtk::MenuItem m_trayMenuExit;
  Gtk::VBox m_vBox1Ctl;
  Gtk::Notebook m_notebookCtl;
  Gtk::Label m_notebookLabel1Ctl;
  Gtk::Label m_notebookLabel2Ctl;
  Gtk::Label m_notebookLabel3Ctl;
  Gtk::HBox m_hBox1Ctl;
  Gtk::HBox m_hBox2Ctl;
  Gtk::HBox m_hBoxCtl;
  Gtk::Table m_tableCtl;
  Gtk::Label m_deviceLabelCtl;
  Gtk::ComboBoxText m_deviceSelectCtl;
  Gtk::Label m_freqLabelCtl;
  Gtk::ComboBoxText m_freqSelectCtl;
  sigc::connection m_timer;
  sigc::slot<bool> m_timerSlot;
  int m_timerNumber;
  bool_t m_bTimerRunning;
  bool_t m_bSelectDeviceFromControl;

protected:
  void enumerateDevices(ADMXRC3_CARD_INFOEX* pCardInfoEx);
  void onAbout();
  bool onDelete(GdkEventAny* pEventInfo);
  void onPopupTrayMenu(guint button, guint32 activateTime);
  void onSelectDevice();
  void onSelectUpdatePeriod();
  bool onTimer(int timerNumber);
  void onTrayActivate();
  void onTrayMenuExit();
  void onTrayMenuRestore();
  bool onWindowStateChanged(GdkEventWindowState* pEventInfo);
  void readSensors();
  void setDevice(unsigned int deviceIndex, bool_t bFromControl = FALSE);
  void setUpdatePeriod(unsigned int index);
  void setNumGraphWnd(unsigned int n);
};
