#include "stdafx.h"
#include "afxwinappex.h"

#ifdef _DEBUG
# define new DEBUG_NEW
#endif

#include <platform.h>
#include <objmodel.h>

#include <math.h>

#include "../../sdk_version.h"
#include "control_id.h"
#include "resource_button.h"
#include "resource_icon.h"
#include "sysmonwnd.h"

IMPLEMENT_DYNAMIC(CSysMonWnd, CWnd)

BEGIN_MESSAGE_MAP(CSysMonWnd, CWnd)
    ON_CBN_SELENDOK(IDCTL_DEVICE_SELECT, OnSelectDevice)
    ON_CBN_SELENDOK(IDCTL_UPDATEPERIOD_SELECT, OnSelectUpdatePeriod)
    ON_NOTIFY(TCN_SELCHANGE, IDCTL_TAB, OnTabChange)
    ON_COMMAND(IDM_RESTORE, OnTrayRestore)
    ON_COMMAND(IDM_EXIT, OnTrayExit)
    ON_WM_CREATE()
    ON_WM_DEVICECHANGE()
    ON_WM_CLOSE()
    ON_WM_SIZE()
    ON_WM_SIZING()
    ON_WM_SYSCOMMAND()
    ON_WM_TIMER()
END_MESSAGE_MAP()

#define ARRAY_LENGTH(x) (sizeof(x) / sizeof((x)[0]))

// Colors for traces (need to add more!)
static const color_t g_traceColors[] = {
  RGB(0xFF, 0x00, 0x00), // Red
  RGB(0x00, 0xFF, 0x00), // Green
  RGB(0xFF, 0xFF, 0x00), // Yellow
  RGB(0x00, 0x00, 0xFF), // Blue
  RGB(0xFF, 0x00, 0xFF), // Magenta
  RGB(0x00, 0xFF, 0xFF), // Cyan
  // And some web colors...
  RGB(0xFA, 0x80, 0x72), // Salmon
  RGB(0x8B, 0x00, 0x8B), // DarkMagenta
  RGB(0x2E, 0x8B, 0x57), // SeaGreen
  RGB(0xB8, 0x86, 0x0B), // DarkGoldenrod
  RGB(0xFF, 0x14, 0x93), // Deep Pink
  RGB(0x80, 0x80, 0x00), // Olive
  RGB(0x00, 0x8B, 0x8B), // DarkCyan
};

// Sensor read periods, selected m_freqSelectCtl
static const UINT g_updatePeriods[] = {
  100, 200, 500, 1000, 2000, 5000, 10000
};

// -------------------------------------------------------------------------
// Start of ISysMonWnd interface
// -------------------------------------------------------------------------

bool_t
CSysMonWnd::addGraphWnd()
{
  IGraphWnd** pp = NULL;
  IGraphWnd* pNew = NULL;
  unsigned int n = m_numGraphWnd + 1;
  CString str;

  pp = new IGraphWnd*[n];
  for (unsigned int i = 0; i < m_numGraphWnd; i++) {
    pp[i] = m_ppGraphWnd[i];
  }
  str.Format(_T("Graph %lu\n"), (unsigned long)n);
  pNew = getFactory().createGraphWnd(this, m_numGraphWnd, m_currentTab == 2);
  pp[n - 1] = pNew;

  delete[] m_ppGraphWnd;
  m_ppGraphWnd = pp;
  setNumGraphWnd(n);

  return TRUE;
}

void
CSysMonWnd::idle()
{
  for (unsigned int i = 0; i < m_numGraphWnd; i++) {
    m_ppGraphWnd[i]->getScope()->idle();
  }
}

bool_t
CSysMonWnd::removeGraphWnd(
  IGraphWnd* pGraphWnd)
{
  for (unsigned int i = 0; i < m_numGraphWnd; i++) {
    if (m_ppGraphWnd[i] == pGraphWnd) {
      return removeGraphWnd(i);
    }
  }

  return FALSE;
}

// -------------------------------------------------------------------------
// End of ISysMonWnd interface
// -------------------------------------------------------------------------

CSysMonWnd::CSysMonWnd()
{
  const TCHAR* pWndClass = NULL;
  int x, y;

  m_deviceIndex = 0;
  m_numGraphWnd = 0;
  m_currentTab = 0;
  ASSERT(ARRAY_LENGTH(g_updatePeriods) >= 4);
  m_updatePeriodIndex = 3;
  m_ppGraphWnd = NULL;
  m_pDeviceInfoWnd = NULL;
  m_pSensorInfoWnd = NULL;
  m_device.m_numDevice = 0;
  m_device.m_hDevice = ADMXRC3_HANDLE_INVALID_VALUE;
  m_device.m_numSensor = 0;
  m_device.m_numAvailableSensor = 0;
  m_device.m_pSensors = NULL;
  m_hIcon = NULL;
  m_hIconTray = NULL;
  m_bMinimizedToTray = FALSE;

  m_bgBrush.CreateSysColorBrush(COLOR_BTNFACE);
  m_font.CreateStockObject(DEFAULT_GUI_FONT);

  m_hIconTray = ::LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_SYSMON));
  m_hIcon = ::LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_SYSMON));
  pWndClass = AfxRegisterWndClass(
    0, ::LoadCursor(NULL, IDC_ARROW), (HBRUSH)m_bgBrush, m_hIcon);
  x = (GetSystemMetrics(SM_CXMAXIMIZED) - m_defaultWidth) / 2;
  y = (GetSystemMetrics(SM_CYMAXIMIZED) - m_defaultHeight) / 2;
  CreateEx(WS_EX_CONTROLPARENT, pWndClass, _T("ADMXRC3 Diagnostics"), WS_SYSMENU | WS_MAXIMIZEBOX | WS_MINIMIZEBOX | WS_SIZEBOX, x, y, m_defaultWidth, m_defaultHeight, NULL, NULL, NULL);

  ShowWindow(SW_SHOW);
}

CSysMonWnd::~CSysMonWnd()
{
}

bool_t
CSysMonWnd::PreTranslateMessage(
  MSG* pMsg)
{
  if (IsDialogMessage(pMsg)) {
    return TRUE;
  } else {
    return CWnd::PreTranslateMessage(pMsg);
  }
}

LRESULT
CSysMonWnd::WindowProc(
  UINT message,
  WPARAM wParam,
  LPARAM lParam)
{
  CPoint point;
  bool_t bSuccess;

  if (message == WM_SYSMON_NOTIFYICON) {
    if (lParam == WM_MOUSEMOVE) {
      return CWnd::WindowProc(message, wParam, lParam);
    }

    switch (lParam) {
    case WM_LBUTTONDBLCLK:
      bSuccess = trayIconHelper(NIM_DELETE, m_hIconTray);
      ASSERT(bSuccess);
      m_bMinimizedToTray = FALSE;
      ShowWindow(SW_RESTORE);
      SetForegroundWindow();
      SetFocus();
      return TRUE;

    case WM_CONTEXTMENU:
      GetCursorPos(&point);
      SetForegroundWindow();
      m_trayMenu.TrackPopupMenu(TPM_RIGHTALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON, point.x, point.y, this);
      return TRUE;

    default:
      break;
    }
  }
  return CWnd::WindowProc(message, wParam, lParam);
}

bool_t
CSysMonWnd::trayIconHelper(
  DWORD dwMessage,
  HICON hIcon)
{
  CString toolTipStr(_T("ADMXRC3 Diagnostics"));
  NOTIFYICONDATA tnd;

  memset(&tnd, 0, sizeof(tnd));
  tnd.cbSize = sizeof(NOTIFYICONDATA);
  tnd.hWnd = (HWND)*this;
  tnd.uID = IDI_LOGO;
  tnd.uFlags = NIF_MESSAGE | NIF_ICON;
  tnd.uCallbackMessage = WM_SYSMON_NOTIFYICON;
  tnd.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
  tnd.hIcon = hIcon;
  _tcscpy_s(tnd.szTip, ARRAY_LENGTH(tnd.szTip), (const TCHAR*)toolTipStr);
  tnd.uVersion = NOTIFYICON_VERSION;
  tnd.dwState = 0;
  tnd.dwStateMask = 0;

  return Shell_NotifyIcon(dwMessage, &tnd);
}

void
CSysMonWnd::readSensors()
{
  SensorInfo* pSensorInfo;
  ADMXRC3_STATUS status;
  ADMXRC3_SENSOR_VALUE sensorValue;

  for (unsigned int i = 0; i < m_device.m_numSensor; i++) {
    pSensorInfo = &m_device.m_pSensors[i];
    if (pSensorInfo->m_bValid) {
      status = ADMXRC3_ReadSensor(m_device.m_hDevice, i, &sensorValue);
      if (ADMXRC3_SUCCESS == status) {
        pSensorInfo->m_current = sensorValue;
      }
    }
  }
  m_pSensorInfoWnd->update();
  for (unsigned int i = 0; i < m_numGraphWnd; i++) {
    m_ppGraphWnd[i]->getScope()->update();
  }
}

void
CSysMonWnd::setNumGraphWnd(
  unsigned int n)
{
  CString str;

  m_numGraphWnd = n;
  for (unsigned int i = 0; i < m_numGraphWnd; i++) {
    m_ppGraphWnd[i]->getToolbar()->numGraphWndChanged();
    m_ppGraphWnd[i]->getToolbar()->numGraphWndChanged();
    str.Format(_T("Graph %lu\n"), (unsigned long)(i + 1));
    m_ppGraphWnd[i]->getToolbar()->setIndex(i);
  }

  RECT r;
  GetClientRect(&r);
  OnSize(SW_SHOW, r.right, r.bottom);
}

bool_t
CSysMonWnd::setDevice(
  unsigned int deviceIndex,
  bool_t bFromControl)
{
  ADMXRC3_STATUS status;
  CString str;
  RECT r;

  GetClientRect(&r);

  if (m_device.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE && deviceIndex == m_deviceIndex) {
    return TRUE;
  }

  if (m_device.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE) {
    ADMXRC3_Close(m_device.m_hDevice);
    m_device.m_hDevice = ADMXRC3_HANDLE_INVALID_VALUE;
    m_device.m_numSensor = 0;
    m_device.m_numAvailableSensor = 0;
    if (NULL != m_device.m_pSensors) {
      delete[] m_device.m_pSensors;
    }
    m_pDeviceInfoWnd->deviceChanged();
    m_pSensorInfoWnd->deviceChanged();
    for (unsigned int i = 0; i < m_numGraphWnd; i++) {
      m_ppGraphWnd[i]->deviceChanged();
    }
  }

  m_deviceIndex = deviceIndex;
  if (!bFromControl) {
    m_deviceSelectCtl.SetCurSel(deviceIndex);
  }

  status = ADMXRC3_OpenEx(m_deviceIndex, TRUE, 0, &m_device.m_hDevice);
  if (ADMXRC3_SUCCESS != status) {
    str.Format(_T("Failed to open device %u: %s"), m_deviceIndex, ADMXRC3_GetStatusString(status, TRUE));
    AfxMessageBox(str, MB_OK | MB_ICONEXCLAMATION);
  } else {
    status = ADMXRC3_GetCardInfoEx(m_device.m_hDevice, &m_device.m_deviceInfo);
    if (ADMXRC3_SUCCESS != status) {
      ADMXRC3_Close(m_device.m_hDevice);
      m_device.m_hDevice = ADMXRC3_HANDLE_INVALID_VALUE;
      str.Format(_T("Failed to get card information: %s"), ADMXRC3_GetStatusString(status, TRUE));
      AfxMessageBox(str, MB_OK | MB_ICONEXCLAMATION);
    } else {
      m_device.m_numSensor = m_device.m_deviceInfo.NumSensor;
      m_device.m_pSensors = new SensorInfo[m_device.m_numSensor];
      ASSERT(NULL != m_device.m_pSensors);
      for (unsigned int i = 0; i < m_device.m_numSensor; i++) {
        status = ADMXRC3_GetSensorInfo(m_device.m_hDevice, i, &m_device.m_pSensors[i].m_info);
        if (ADMXRC3_SUCCESS != status) {
          m_device.m_pSensors[i].m_bValid = FALSE;
        } else {
          m_device.m_pSensors[i].m_bValid = TRUE;
          switch (m_device.m_pSensors[i].m_info.DataType) {
        case ADMXRC3_DATA_BOOL:
        case ADMXRC3_DATA_DOUBLE:
        case ADMXRC3_DATA_INT32:
        case ADMXRC3_DATA_UINT32:
          m_device.m_pSensors[i].m_bKnownDataType = TRUE;
          m_device.m_pSensors[i].m_exponent = powf(10.0f, (float)m_device.m_pSensors[i].m_info.Exponent);
          m_device.m_numAvailableSensor++;
          m_device.m_pSensors[i].m_traceColor = RGB(0xFF, 0xFF, 0xFF);
          if (i < ARRAY_LENGTH(g_traceColors)) {
            m_device.m_pSensors[i].m_traceColor = g_traceColors[i];
          }
          break;

        default:
          /* Don't understand this datatype */
          m_device.m_pSensors[i].m_bKnownDataType = FALSE;
          }
        }
      }
      m_pDeviceInfoWnd->deviceChanged();
      m_pSensorInfoWnd->deviceChanged();
      for (unsigned int i = 0; i < m_numGraphWnd; i++) {
        m_ppGraphWnd[i]->deviceChanged();
      }
    }
  }

  if (m_device.m_numAvailableSensor) {
    SetTimer(m_timerId, g_updatePeriods[m_updatePeriodIndex], NULL);
  }

  OnSize(SW_SHOW, r.right, r.bottom);

  return TRUE;
}

bool_t
CSysMonWnd::removeGraphWnd(
  unsigned int index)
{
  CString str;
  unsigned int n = m_numGraphWnd;
  IGraphWnd* pGraphWnd;

  pGraphWnd = m_ppGraphWnd[index];
  pGraphWnd->getNativeControl().DestroyWindow();
  delete pGraphWnd;

  for (unsigned int i = index + 1; i < n; i++) {
    m_ppGraphWnd[i - 1] = m_ppGraphWnd[i];
  }
  setNumGraphWnd(n - 1);

  return TRUE;
}

void
CSysMonWnd::enumerateDevices(
  ADMXRC3_CARD_INFOEX* pCardInfoEx)
{
  ADMXRC3_CARD_INFOEX cardInfoEx;
  ADMXRC3_STATUS status;
  ADMXRC3_HANDLE hDevice;
  const TCHAR* pModelName;
  unsigned int i, index = 0;
  BOOL bFound = FALSE, bNumDeviceChanged;
  CString str;

  m_deviceSelectCtl.Clear();
  for (i = 0; i < 0x100; i++) {
    status = ADMXRC3_OpenEx(i, TRUE, 0, &hDevice);
    if (ADMXRC3_SUCCESS == status) {
      status = ADMXRC3_GetCardInfoEx(hDevice, &cardInfoEx);
      if (ADMXRC3_SUCCESS == status) {
        pModelName = CAdmxrc3Help::getModelName(cardInfoEx.Model);
        if (NULL == pModelName) {
          str.Format(_T("Index %u (Unknown model %lu (0x%lx)) SN #%lu"), i, (unsigned long)cardInfoEx.Model, (unsigned long)cardInfoEx.Model, (unsigned long)cardInfoEx.SerialNumber);
        } else {
          str.Format(_T("Index %u %s SN #%lu"), i, (const TCHAR*)pModelName, (unsigned long)cardInfoEx.SerialNumber);
        }
      } else {
        str.Format(_T("Index %u (Could not determine model)"), i);
      }
      ADMXRC3_Close(hDevice);
      if (NULL != pCardInfoEx && cardInfoEx.Model == pCardInfoEx->Model && cardInfoEx.SerialNumber == pCardInfoEx->SerialNumber) {
        index = i;
        bFound = TRUE;
      }
    } else {
      if (ADMXRC3_DEVICE_NOT_FOUND == status) {
        break;
      } else {
        str.Format(_T("Index %u (Could not open)"), i);
      }
    }
    m_deviceSelectCtl.AddString(str);
  }
  bNumDeviceChanged = (m_device.m_numDevice != i);
  m_device.m_numDevice = i;
  if (0 == m_device.m_numDevice) {
    m_deviceSelectCtl.AddString(_T("No devices available"));
  }
  if (bFound) {
    m_deviceSelectCtl.SetCurSel(index);
  }
  m_deviceSelectCtl.EnableWindow(0 != m_device.m_numDevice);
  if (bNumDeviceChanged) {
    m_pDeviceInfoWnd->numDeviceChanged();
    m_pSensorInfoWnd->numDeviceChanged();
  }
}

void
CSysMonWnd::OnClose()
{
  if (m_bMinimizedToTray) {
    trayIconHelper(NIM_DELETE, m_hIconTray);
    m_bMinimizedToTray = FALSE;
  }

  DestroyWindow();

  if (NULL != m_ppGraphWnd) {
    for (unsigned int i = 0; i < m_numGraphWnd; i++) {
      m_ppGraphWnd[i]->getNativeControl().DestroyWindow();
      delete m_ppGraphWnd[i];
      m_ppGraphWnd[i] = NULL;
    }
    delete[] m_ppGraphWnd;
    m_ppGraphWnd = NULL;
  }
  if (NULL != m_pDeviceInfoWnd) {
      m_pDeviceInfoWnd->getNativeControl().DestroyWindow();
      delete m_pDeviceInfoWnd;
  }
  if (NULL != m_pSensorInfoWnd) {
      m_pSensorInfoWnd->getNativeControl().DestroyWindow();
      delete m_pSensorInfoWnd;
  }
  if (NULL != m_device.m_pSensors) {
    delete[] m_device.m_pSensors;
    m_device.m_pSensors = NULL;
  }
  if (ADMXRC3_HANDLE_INVALID_VALUE != m_device.m_hDevice) {
    ADMXRC3_Close(m_device.m_hDevice);
    m_device.m_hDevice = ADMXRC3_HANDLE_INVALID_VALUE;
  }
  if (NULL != m_hIcon) {
    DestroyIcon(m_hIcon);
    m_hIcon = NULL;
  }
  if (NULL != m_hIconTray) {
    DestroyIcon(m_hIconTray);
    m_hIconTray = NULL;
  }
}

int
CSysMonWnd::OnCreate(
  LPCREATESTRUCT lpCreateStruct)
{
  RECT r = { 0, 0, 0, 0 };

  if (CWnd::OnCreate(lpCreateStruct) == -1) {
    return -1;
  }

  m_deviceLabelCtl.Create(_T("Device"), WS_VISIBLE | WS_CHILD | SS_RIGHT, r, this, (UINT)IDC_STATIC);
  m_deviceLabelCtl.SetFont(&m_font);
  
  m_deviceSelectCtl.Create(WS_VISIBLE | WS_CHILD | WS_VSCROLL | WS_GROUP | WS_TABSTOP | CBS_DROPDOWNLIST, r, this, (UINT)IDCTL_DEVICE_SELECT);
  m_deviceSelectCtl.SetFont(&m_font);

  m_freqLabelCtl.Create(_T("Update period"), WS_VISIBLE | WS_CHILD | SS_RIGHT, r, this, (UINT)IDC_STATIC);
  m_freqLabelCtl.SetFont(&m_font);

  m_freqSelectCtl.Create(WS_VISIBLE | WS_CHILD | WS_VSCROLL | WS_TABSTOP | CBS_DROPDOWNLIST, r, this, (UINT)IDCTL_UPDATEPERIOD_SELECT);
  m_freqSelectCtl.SetFont(&m_font);
  for (unsigned int i = 0; i < ARRAY_LENGTH(g_updatePeriods); i++) {
    CString str;

    if (g_updatePeriods[i] >= 1000) {
      str.Format(_T("%.f s"), (double)g_updatePeriods[i] / 1000.0);
    } else {
      str.Format(_T("%lu ms"), (unsigned long)g_updatePeriods[i]);
    }
    m_freqSelectCtl.AddString(str);
  }
  m_freqSelectCtl.SetCurSel(m_updatePeriodIndex);

  m_tabCtl.CreateEx(0, TCS_FLATBUTTONS | TCS_TABS | WS_CHILD | WS_TABSTOP, r, this, IDCTL_TAB);
  m_tabCtl.SetFont(&m_font);
  m_tabCtl.InsertItem(0, _T("Device Information"));
  m_tabCtl.InsertItem(1, _T("Sensor Information"));
  m_tabCtl.InsertItem(2, _T("Sensor Readout"));
  m_tabCtl.SetCurSel(0);
  m_tabCtl.ShowWindow(SW_SHOW);

  m_pDeviceInfoWnd = getFactory().createDeviceInfoWnd(this, TRUE);

  m_pSensorInfoWnd = getFactory().createSensorInfoWnd(this, FALSE);
  
  m_trayMenu.CreatePopupMenu();
  m_trayMenu.AppendMenu(MF_STRING, IDM_RESTORE, _T("Restore"));
  m_trayMenu.AppendMenu(MF_STRING, IDM_EXIT, _T("Exit"));

  CMenu* pSystemMenu = GetSystemMenu(FALSE);
  pSystemMenu->AppendMenu(MF_SEPARATOR);
  pSystemMenu->AppendMenu(MF_STRING, IDM_SYSMENU_ABOUT, _T("About ADMXRC3 Diagnostics"));

  addGraphWnd();
  
  enumerateDevices(NULL);
  if (0 != m_device.m_numDevice) {
    m_deviceSelectCtl.SetCurSel(0);
    setDevice(0, TRUE);
  }

  OnSize(SW_SHOW, r.right, r.bottom);

  return 0;
}

BOOL
CSysMonWnd::OnDeviceChange(UINT /* nEventType */, DWORD_PTR /* eventData */)
{
  enumerateDevices(&m_device.m_deviceInfo);
  return TRUE;
}

void
CSysMonWnd::OnSelectDevice()
{
  setDevice(m_deviceSelectCtl.GetCurSel(), TRUE);
}

void
CSysMonWnd::OnSelectUpdatePeriod()
{
  int index = m_freqSelectCtl.GetCurSel();

  if (m_updatePeriodIndex < index && m_device.m_numAvailableSensor) {
    KillTimer(m_timerId);
  }
  m_updatePeriodIndex = index;
  if (m_device.m_numAvailableSensor) {
    SetTimer(m_timerId, g_updatePeriods[m_updatePeriodIndex], NULL);
  }
  for (unsigned int i = 0; i < m_numGraphWnd; i++) {
    m_ppGraphWnd[i]->getScope()->updatePeriodChanged();
  }
}

void
CSysMonWnd::OnSize(
  UINT nType,
  int cx,
  int cy)
{
  RECT r1, r2;
  POINT p;

  CWnd::OnSize(nType, cx, cy);

  if (nType == SIZE_MINIMIZED) {
    ShowWindow(SW_HIDE);
    trayIconHelper(NIM_ADD, m_hIconTray);
    trayIconHelper(NIM_SETVERSION, m_hIconTray);
    m_bMinimizedToTray = TRUE;
    return;
  }

  m_deviceLabelCtl.MoveWindow(10, 14, 40, 20);
  m_deviceSelectCtl.MoveWindow(60, 10, 250, 120);

  m_freqLabelCtl.MoveWindow(330, 14, 80, 20);
  m_freqSelectCtl.MoveWindow(420, 10, 80, 120);

  m_tabCtl.MoveWindow(10, 45, cx - 20, cy - 55);

  m_pDeviceInfoWnd->getNativeControl().MoveWindow(20, 75, cx - 40, cy - 95);

  m_pSensorInfoWnd->getNativeControl().MoveWindow(20, 75, cx - 40, cy - 95);

  r1.left = 20;
  r1.top = 75;
  r1.right = cx - 20;
  r1.bottom = cy - 20;
  p.x = (r1.right + r1.left) >> 1;
  p.y = (r1.top + r1.bottom) >> 1;
  if (NULL != m_ppGraphWnd) {
    switch (m_numGraphWnd) {
    case 1:
      r2.left = r1.left;
      r2.top = r1.top;
      r2.right = r1.right;
      r2.bottom = r1.bottom;
      m_ppGraphWnd[0]->getNativeControl().MoveWindow(r2.left, r2.top, r2.right - r2.left, r2.bottom - r2.top);
      break;

    case 2:
      r2.left = r1.left;
      r2.top = r1.top;
      r2.right = p.x - 5;
      r2.bottom = r1.bottom;
      m_ppGraphWnd[0]->getNativeControl().MoveWindow(r2.left, r2.top, r2.right - r2.left, r2.bottom - r2.top);
      r2.left = p.x + 5;
      r2.right = r1.right;
      m_ppGraphWnd[1]->getNativeControl().MoveWindow(r2.left, r2.top, r2.right - r2.left, r2.bottom - r2.top);
      break;

    case 3:
      r2.left = r1.left;
      r2.top = r1.top;
      r2.right = p.x - 5;
      r2.bottom = p.y - 5;
      m_ppGraphWnd[0]->getNativeControl().MoveWindow(r2.left, r2.top, r2.right - r2.left, r2.bottom - r2.top);
      r2.left = p.x + 5;
      r2.right = r1.right;
      m_ppGraphWnd[1]->getNativeControl().MoveWindow(r2.left, r2.top, r2.right - r2.left, r2.bottom - r2.top);
      r2.left = r1.left;
      r2.top = p.y + 5;
      r2.right = r1.right;
      r2.bottom = r1.bottom;
      m_ppGraphWnd[2]->getNativeControl().MoveWindow(r2.left, r2.top, r2.right - r2.left, r2.bottom - r2.top);
      break;

    case 4:
      r2.left = r1.left;
      r2.top = r1.top;
      r2.right = p.x - 5;
      r2.bottom = p.y - 5;
      m_ppGraphWnd[0]->getNativeControl().MoveWindow(r2.left, r2.top, r2.right - r2.left, r2.bottom - r2.top);
      r2.left = p.x + 5;
      r2.right = r1.right;
      m_ppGraphWnd[1]->getNativeControl().MoveWindow(r2.left, r2.top, r2.right - r2.left, r2.bottom - r2.top);
      r2.left = r1.left;
      r2.top = p.y + 5;
      r2.right = p.x - 5;
      r2.bottom = r1.bottom;
      m_ppGraphWnd[2]->getNativeControl().MoveWindow(r2.left, r2.top, r2.right - r2.left, r2.bottom - r2.top);
      r2.left = p.x + 5;
      r2.right = r1.right;
      m_ppGraphWnd[3]->getNativeControl().MoveWindow(r2.left, r2.top, r2.right - r2.left, r2.bottom - r2.top);
      break;

    default:
      ASSERT(FALSE);
    }
  }
}

void
CSysMonWnd::OnSizing(
  UINT nSide,
  RECT* pRect)
{
  int w = pRect->right - pRect->left;
  int h = pRect->bottom - pRect->top;

  if (w < m_minWidth) {
    if (nSide == WMSZ_BOTTOMLEFT || nSide == WMSZ_LEFT || nSide == WMSZ_TOPLEFT) {
      pRect->left = pRect->right - m_minWidth;
    } else if (nSide == WMSZ_BOTTOMRIGHT || nSide == WMSZ_RIGHT || nSide == WMSZ_TOPRIGHT) {
      pRect->right = pRect->left + m_minWidth;
    } else {
      ASSERT(FALSE);
    }
  }
  if (h < m_minHeight) {
    if (nSide == WMSZ_BOTTOMLEFT || nSide == WMSZ_BOTTOM || nSide == WMSZ_BOTTOMRIGHT) {
      pRect->bottom = pRect->top + m_minHeight;
    } else if (nSide == WMSZ_TOPLEFT || nSide == WMSZ_TOP || nSide == WMSZ_TOPRIGHT) {
      pRect->top = pRect->bottom - m_minWidth;
    } else {
      ASSERT(FALSE);
    }
  }
}

void
CSysMonWnd::OnSysCommand(UINT nID, LPARAM lParam)
{
  if (nID == IDM_SYSMENU_ABOUT) {
    unsigned int version[4] = { ADMXRC_SDK_VERSION_NUM };
    CString str;

    str.Format(_T("(C) Copyright 2010 Alpha Data\n\nVersion %u.%u.%u.%u"), version[0], version[1], version[2], version[3]);
    MessageBox(str, _T("About ADMXRC3 Diagnostics"), MB_OK | MB_ICONINFORMATION);
  } else {
    CWnd::OnSysCommand(nID, lParam);
  }
}

void
CSysMonWnd::OnTabChange(
  NMHDR* /* pNotifyStruct */,
  LRESULT* /* pResult */)
{
  int i = m_tabCtl.GetCurSel();

  if (i != m_currentTab) {
    m_currentTab = i;
    m_pDeviceInfoWnd->getNativeControl().ShowWindow(i == 0 ? SW_SHOW : SW_HIDE);
    m_pSensorInfoWnd->getNativeControl().ShowWindow(i == 1 ? SW_SHOW : SW_HIDE);
    for (unsigned int j = 0; j < m_numGraphWnd; j++) {
      m_ppGraphWnd[j]->getNativeControl().ShowWindow(i == 2 ? SW_SHOW : SW_HIDE);
    }
  }
}

void
CSysMonWnd::OnTimer(
  UINT_PTR nTimerId)
{
  if (nTimerId == m_timerId) {
    if (m_device.m_numAvailableSensor) {
      SetTimer(m_timerId, g_updatePeriods[m_updatePeriodIndex], NULL);
      readSensors();
    }
  }
}

void
CSysMonWnd::OnTrayExit()
{
  DestroyWindow();
}

void
CSysMonWnd::OnTrayRestore()
{
  bool_t bSuccess;

  bSuccess = trayIconHelper(NIM_DELETE, m_hIconTray);
  ASSERT(bSuccess);
  m_bMinimizedToTray = FALSE;
  ShowWindow(SW_RESTORE);
  SetForegroundWindow();
  SetFocus();
}
