--==============================================================================
--
--     File Name: adb3_target_pkg.vhd
--       Version: 0.1
--          Date: 06 Nov 2009
--         Model: Synthesisable
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: Package adb3_target_pkg
--
--   Description: Package of components used in target FPGAs using ADB3.
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2009 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 06 Nov 09: Initial design
--
--==============================================================================

--
-- Exported routines (parenthesis indicated number of overloads):
--
--   dmask_width
--
--
-- Exported components:
--
--   mptl_if_target_wrap
--   mptl_if_target_wrap_m
--   mptl_if_target_sim
--   mptl_if_target_syn
--   mptl_if_target_syn_m
--   mptl64par_interface_target_6tl
--   mptl64par_interface_target_6tl_master
--   mptl128_interface_target_6t1
--   mptl128_interface_target_6t1_master
--
--   adb3_ocp_write_fifo_interface
--   adb3_ocp_read_fifo_interface
--   adb3_ocp_master_bus_interface
--   adb3_ocp_simple_bus_if
--   adb3_ocp_reg_split
--   adb3_ocp_mux
--   adb3_ocp_cross_ck_dom
--

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_types_pkg.all;
use work.adb3_target_inc_pkg.all;

package adb3_target_pkg is

  -- Define total size of target Direct Slave address space: 32 bits = 4 GB
  constant DS_ADDR_WIDTH : natural range 1 to ADB3_OCP_ADDR_WIDTH := 32;
  subtype ds_addr_s is std_logic_vector(DS_ADDR_WIDTH-1 downto 0);

  -- Define total size of target Direct Master address space: 64 bits
  constant DM_ADDR_WIDTH : natural range 1 to ADB3_OCP_ADDR_WIDTH := 32;
  subtype dm_addr_s is std_logic_vector(DM_ADDR_WIDTH-1 downto 0);

  -- Define total size of target DMA address space: 64 bits
  constant DMA_ADDR_WIDTH : natural range 1 to ADB3_OCP_ADDR_WIDTH := 32;
  subtype dma_addr_s is std_logic_vector(DMA_ADDR_WIDTH-1 downto 0);


  -- Define adb3_ocp_reg_split types
  type ds_addr_range_t is record
    addr_base : ds_addr_s;
    addr_mask : ds_addr_s;
  end record;

  type ds_addr_range_table_t is array (natural range <>) of ds_addr_range_t;


  function ds_vec_width(
    constant x : in ds_addr_s)
  return natural;

  function make_defined_s2m(
    constant x : in adb3_ocp_s2mT)
  return adb3_ocp_s2mT;

  function make_defined(
    constant x : in std_logic_vector)
  return std_logic_vector;


  component mptl_if_target_wrap
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   adb3_ocp_m2sT;
    direct_slave_s2m         : in    adb3_ocp_s2mT;
    dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to MAX_DMA_CHANNELS-1);
    dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to MAX_DMA_CHANNELS-1);
    -- MPTL Interface
    mptl_t2b                 : out   mptl_pins_t;
    mptl_b2t                 : in    mptl_pins_t;
    mptl_clk                 : in    mptl_clk_t;
    -- Bridge to Target Sideband Signals
    mptl_target_configured_n : out   std_logic;
    mptl_target_gtp_online_n : out   std_logic;
    mptl_bridge_gtp_online_n : in    std_logic;
    -- MPTL Sideband Interface
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

  component mptl_if_target_wrap_m
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   adb3_ocp_m2sT;
    direct_slave_s2m         : in    adb3_ocp_s2mT;
    dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to MAX_DMA_CHANNELS-1);
    dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to MAX_DMA_CHANNELS-1);
    direct_masters_m2s       : in    adb3_ocp_m2sT_array(0 to MAX_DM_CHANNELS-1);
    direct_masters_s2m       : out   adb3_ocp_s2mT_array(0 to MAX_DM_CHANNELS-1);
    -- MPTL Interface
    mptl_t2b                 : out   mptl_pins_t;
    mptl_b2t                 : in    mptl_pins_t;
    mptl_clk                 : in    mptl_clk_t;
    -- Bridge to Target Sideband Signals
    mptl_target_configured_n : out   std_logic;
    mptl_target_gtp_online_n : out   std_logic;
    mptl_bridge_gtp_online_n : in    std_logic;
    -- MPTL Sideband Interface
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

  component mptl_if_target_sim
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   adb3_ocp_m2sT;
    direct_slave_s2m         : in    adb3_ocp_s2mT;
    dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to MAX_DMA_CHANNELS-1);
    dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to MAX_DMA_CHANNELS-1);
    direct_masters_m2s       : in    adb3_ocp_m2sT_array(0 to MAX_DM_CHANNELS-1);
    direct_masters_s2m       : out   adb3_ocp_s2mT_array(0 to MAX_DM_CHANNELS-1);
    -- MPTL Interface
    mptl_b2t                 : in    mptl_pins_t;
    mptl_t2b                 : out   mptl_pins_t;
    mptl_clk                 : in    mptl_clk_t;
    -- Bridge to Target Sideband Signals
    mptl_target_configured_n : out   std_logic;
    mptl_target_gtp_online_n : out   std_logic;
    mptl_bridge_gtp_online_n : in    std_logic;
    -- MPTL Sideband Interface
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

  component mptl_if_target_syn
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   adb3_ocp_m2sT;
    direct_slave_s2m         : in    adb3_ocp_s2mT;
    dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to MAX_DMA_CHANNELS-1);
    dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to MAX_DMA_CHANNELS-1);
    -- MPTL Interface
    mptl_b2t                 : in    mptl_pins_t;
    mptl_t2b                 : out   mptl_pins_t;
    mptl_clk                 : in    mptl_clk_t;
    -- Bridge to Target Sideband Signals
    mptl_target_configured_n : out   std_logic;
    mptl_target_gtp_online_n : out   std_logic;
    mptl_bridge_gtp_online_n : in    std_logic;
    -- MPTL Sideband Interface
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

  component mptl_if_target_syn_m
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   adb3_ocp_m2sT;
    direct_slave_s2m         : in    adb3_ocp_s2mT;
    dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to MAX_DMA_CHANNELS-1);
    dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to MAX_DMA_CHANNELS-1);
    direct_masters_m2s       : in    adb3_ocp_m2sT_array(0 to MAX_DM_CHANNELS-1);
    direct_masters_s2m       : out   adb3_ocp_s2mT_array(0 to MAX_DM_CHANNELS-1);
    -- MPTL Interface
    mptl_b2t                 : in    mptl_pins_t;
    mptl_t2b                 : out   mptl_pins_t;
    mptl_clk                 : in    mptl_clk_t;
    -- Bridge to Target Sideband Signals
    mptl_target_configured_n : out   std_logic;
    mptl_target_gtp_online_n : out   std_logic;
    mptl_bridge_gtp_online_n : in    std_logic;
    -- MPTL Sideband Interface
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

  component mptl64par_interface_target_6tl
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   adb3_ocp_m2sT;
    direct_slave_s2m         : in    adb3_ocp_s2mT;
    dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to 1);
    dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to 1);
    -- MPTL Interface
    mptl_tx                  : out   mptl_pins_t;
    mptl_rx                  : in    mptl_pins_t;
    mptl_clk                 : in    mptl_clk_t;
    -- Bridge to Target Sideband Signals
    mptl_target_configured_n : out   std_logic;
    mptl_target_gtp_online_n : out   std_logic;
    mptl_bridge_gtp_online_n : in    std_logic;
    -- MPTL Sideband Interface
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

  component mptl64par_interface_target_6tl_master
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   adb3_ocp_m2sT;
    direct_slave_s2m         : in    adb3_ocp_s2mT;
    dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to 1);
    dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to 1);
    direct_master_m2s        : in    adb3_ocp_m2sT;
    direct_master_s2m        : out   adb3_ocp_s2mT;
    -- MPTL Interface
    mptl_tx                  : out   mptl_pins_t;
    mptl_rx                  : in    mptl_pins_t;
    mptl_clk                 : in    mptl_clk_t;
    -- Bridge to Target Sideband Signals
    mptl_target_configured_n : out   std_logic;
    mptl_target_gtp_online_n : out   std_logic;
    mptl_bridge_gtp_online_n : in    std_logic;
    -- MPTL Sideband Interface
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

  component mptl128_interface_target_6t1
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   adb3_ocp_m2sT;
    direct_slave_s2m         : in    adb3_ocp_s2mT;
    dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to 3);
    dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to 3);
    -- MPTL Interface
    mptl_tx                  : out   mptl_pins_t;
    mptl_rx                  : in    mptl_pins_t;
    mptl_clk                 : in    mptl_clk_t;
    -- Bridge to Target Sideband Signals
    mptl_target_configured_n : out   std_logic;
    mptl_target_gtp_online_n : out   std_logic;
    mptl_bridge_gtp_online_n : in    std_logic;
    -- MPTL64 Sideband Interface
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

  component mptl128_interface_target_6t1_master
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   adb3_ocp_m2sT;
    direct_slave_s2m         : in    adb3_ocp_s2mT;
    dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to 3);
    dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to 3);
    direct_master_m2s        : in    adb3_ocp_m2sT;
    direct_master_s2m        : out   adb3_ocp_s2mT;
    -- MPTL Interface
    mptl_tx                  : out   mptl_pins_t;
    mptl_rx                  : in    mptl_pins_t;
    mptl_clk                 : in    mptl_clk_t;
    -- Bridge to Target Sideband Signals
    mptl_target_configured_n : out   std_logic;
    mptl_target_gtp_online_n : out   std_logic;
    mptl_bridge_gtp_online_n : in    std_logic;
    -- MPTL64 Sideband Interface
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

  component adb3_ocp_master_bus_if
  generic(
    addr_width : natural range 1 to ADB3_OCP_ADDR_WIDTH;
    data_width : natural range 8 to ADB3_OCP_DATA_WIDTH);
  port(
    -- OCP Interface
    ocp_rst : in    std_logic;
    ocp_clk : in    std_logic;
    ocp_m2s : out   adb3_ocp_m2sT;
    ocp_s2m : in    adb3_ocp_s2mT;
    -- Simple bus interface
    go      : in    std_logic;
    d       : in    std_logic_vector(data_width-1 downto 0);
    q       : out   std_logic_vector(data_width-1 downto 0);
    a       : in    std_logic_vector(addr_width-1 downto 0);
    w       : in    std_logic;
    be      : in    std_logic_vector((data_width/8)-1 downto 0);
    idle    : out   std_logic);
  end component;

  component adb3_ocp_simple_bus_if
  generic(
    addr_width   : natural range 1 to ADB3_OCP_ADDR_WIDTH;
    data_width   : natural range 8 to ADB3_OCP_DATA_WIDTH;
    read_latency : natural := 1);
  port(
    -- OCP Interface
    ocp_rst : in    std_logic;
    ocp_clk : in    std_logic;
    ocp_m2s : in    adb3_ocp_m2sT;
    ocp_s2m : out   adb3_ocp_s2mT;
    -- Simple bus interface
    d       : out   std_logic_vector(data_width-1 downto 0);
    q       : in    std_logic_vector(data_width-1 downto 0);
    a       : out   std_logic_vector(addr_width-1 downto 0);
    w       : out   std_logic;
    r       : out   std_logic;
    we      : out   std_logic_vector((data_width/8)-1 downto 0));
  end component;

  component adb3_ocp_reg_split
  generic(
    addr_range_table : ds_addr_range_table_t);
  port(
    -- OCP Interface
    ocp_rst     : in    std_logic;
    ocp_clk     : in    std_logic;
    master_m2s  : in    adb3_ocp_m2sT;
    master_s2m  : out   adb3_ocp_s2mT;
    slaves_m2s  : out   adb3_ocp_m2sT_array(0 to addr_range_table'length-1);
    slaves_s2m  : in    adb3_ocp_s2mT_array(0 to addr_range_table'length-1);
    -- Status interface
    w_addr_oor  : out   std_logic;
    r_addr_oor  : out   std_logic);
  end component;

  component adb3_ocp_mux
  generic (
    mux_inputs      : natural := 3);
--    round_robin_arb : boolean := false;
--    use_timeout     : boolean := false;
--    timeout_bits    : natural := 16);
  port(
    ocp_rst       : in    std_logic;
    ocp_clk       : in    std_logic;
    -- Upstream OCP interfaces
    ocp_m2s       : in    adb3_ocp_m2sT_array(0 to mux_inputs-1);
    ocp_s2m       : out   adb3_ocp_s2mT_array(0 to mux_inputs-1);
    -- Downstream OCP interface
    ocp_m2s_memif : out   adb3_ocp_m2sT;
    ocp_s2m_memif : in    adb3_ocp_s2mT;
    -- Status
    status_w      : out   std_logic_vector(mux_inputs-1 downto 0);
    status_r      : out   std_logic_vector(mux_inputs-1 downto 0));
  end component;

  component adb3_ocp_cross_clk_dom
  port(
    -- Upstream OCP interface
    master_rst    : in    std_logic;
    master_clk    : in    std_logic;
    master_m2s    : in    adb3_ocp_m2sT;
    master_s2m    : out   adb3_ocp_s2mT;
    master_bl_err : out   std_logic;
    -- Downstream OCP interface
    slave_rst     : in    std_logic;
    slave_clk     : in    std_logic;
    slave_m2s     : out   adb3_ocp_m2sT;
    slave_s2m     : in    adb3_ocp_s2mT;
    slave_bl_err  : out   std_logic);
  end component;

end;


package body adb3_target_pkg is

  -- Convert active address mask width to natural
  function ds_vec_width(
    constant x : in ds_addr_s)
  return natural is
    variable n : natural;
  begin
    n := 0;
    for j in 1 to DS_ADDR_WIDTH  loop
      if (x(j-1) = '1') then
        n := j;
      end if;
    end loop;

    return n;
  end;

  -- Make sure OCP s2m record has defined data for MPTL serial interface simulation
  function make_defined_s2m(
    constant x : in adb3_ocp_s2mT)
  return adb3_ocp_s2mT is
    variable m : adb3_ocp_s2mT;  -- New OCP record
  begin
    m.CmdAccept  := x.CmdAccept;
    m.DataAccept := x.DataAccept;
    m.Resp       := x.Resp;
    m.Data       := make_defined(x.Data);
    m.Tag        := x.Tag;
    return m;
  end;

  -- Convert any non '0' or '1' bits of std_logic_vector to '0'
  function make_defined(
    constant x : in std_logic_vector)
  return std_logic_vector is
    variable n : natural := x'length;               -- Size of x in bits (min size of 1)
    variable m : std_logic_vector(n - 1 downto 0);  -- New vector of size n bits
  begin
    for i in 0 to (n - 1) loop
      if (x(i) = '0')or(x(i) = '1') then
        m(i) := x(i);
      else
        m(i) := '0';
      end if;
    end loop;

    return m;
  end;

end;