--==============================================================================
--
--     File Name: rst_pkg.vhd
--       Version: 0.1
--          Date: 10 Jun 2010
--         Model: RTL (SYNTHESISABLE)
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: Reset generation package
--
--   Description: Reset generation package and entities
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2010 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 10 Jun 10: Initial design
--
--
--==============================================================================
library ieee;
use ieee.std_logic_1164.all;

package rst_pkg is

  ----------------------
  -- Reset components --
  ----------------------

  component rst_sync
  port(
    async_rst  : in    std_logic;
    clk        : in    std_logic;
    master_rst : out   std_logic);
  end component;

end package rst_pkg;


package body rst_pkg is

end package body;


------------------
-- Reset blocks --
------------------

--
-- rst_sync
-- produce master reset from asynchronous reset input
-- master reset is applied asynchronously and removed synchronously
--
library ieee;
use ieee.std_logic_1164.all;

library work;
use work.rst_pkg.all;

entity rst_sync is
port(
  async_rst  : in    std_logic;   -- asynchronous reset input
  clk        : in    std_logic;   -- clock domain for output reset off
  master_rst : out   std_logic);  -- asynchronous on, synchronous off master reset
end entity rst_sync;

architecture syn of rst_sync is
  signal rst_reg_d1 : std_logic;
  signal rst_reg_d2 : std_logic;
begin
  rst_reg_p : process(async_rst,clk)
  begin
    if (async_rst = '1') then
      rst_reg_d1 <= '1';
      rst_reg_d2 <= '1';
    elsif rising_edge(clk) then
      rst_reg_d1 <= '0';
      rst_reg_d2 <= rst_reg_d1;
    end if;
  end process;

  master_rst <= rst_reg_d2;
end architecture;
