--==============================================================================
--
--     File Name: mptl_if_bridge_wrap.vhd
--       Version: 0.1
--          Date: 23 Mar 2010
--         Model: sim (Not Synthesisable)
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: mptl_if_bridge_wrap wrapper
--
--   Description: Selects between the different versions of bridge mptl i/f
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2010 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 23 Mar 2010: Initial design
--
--==============================================================================


library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_types_pkg.all;
use work.adb3_target_inc_pkg.all;
use work.adb3_target_pkg.all;
use work.adb3_target_tb_pkg.all;

entity mptl_if_bridge_wrap is
port(
  -- OCP Interface
  ocp_clk                  : in    std_logic;
  direct_slave_m2s         : in    adb3_ocp_m2sT;
  direct_slave_s2m         : out   adb3_ocp_s2mT;
  dma_channels_m2s         : in    adb3_ocp_m2sT_array(0 to MAX_DMA_CHANNELS-1);
  dma_channels_s2m         : out   adb3_ocp_s2mT_array(0 to MAX_DMA_CHANNELS-1);
  direct_masters_m2s       : out   adb3_ocp_m2sT_array(0 to MAX_DM_CHANNELS-1);
  direct_masters_s2m       : in    adb3_ocp_s2mT_array(0 to MAX_DM_CHANNELS-1);
  -- MPTL Interface
  mptl_b2t                 : out   mptl_pins_t;
  mptl_t2b                 : in    mptl_pins_t;
  mptl_clk                 : in    mptl_clk_t;
  -- Bridge to Target Sideband Signals
  mptl_target_configured_n : in    std_logic;
  mptl_target_gtp_online_n : in    std_logic;
  mptl_bridge_gtp_online_n : out   std_logic;
  -- MPTL Sideband Interface
  gpio_b2t                 : in    std_logic_vector(15 downto 0);
  gpio_t2b                 : out   std_logic_vector(15 downto 0);
  mptl_status              : out   std_logic_vector(63 downto 0);
  mptl_config              : in    std_logic_vector(31 downto 0));
end;


architecture sim of mptl_if_bridge_wrap is

  -- Configuration/online signals
  signal target_configured : std_logic;
  signal target_gtp_online : std_logic;
  signal bridge_gtp_online : std_logic;

begin

  -- Connect MPTL sideband inputs
  target_configured <= not(mptl_target_configured_n);
  target_gtp_online <= not(mptl_target_gtp_online_n);

  inst_sim_ocp_g : if (TARGET_USE = SIM_OCP) generate
    -- direct OCP-OCP simulation interface
    mptl_if_bridge_sim_i : mptl_if_bridge_sim
    port map(
      -- OCP Interface
      ocp_clk            => ocp_clk,                                    -- in    std_logic
      direct_slave_m2s   => direct_slave_m2s,                           -- in    adb3_ocp_m2sT
      direct_slave_s2m   => direct_slave_s2m,                           -- out   adb3_ocp_s2mT
      dma_channels_m2s   => dma_channels_m2s(0 to MAX_DMA_CHANNELS-1),  -- in    adb3_ocp_m2sT_array(0 to MAX_DMA_CHANNELS-1)
      dma_channels_s2m   => dma_channels_s2m(0 to MAX_DMA_CHANNELS-1),  -- out   adb3_ocp_s2mT_array(0 to MAX_DMA_CHANNELS-1)
      direct_masters_m2s => direct_masters_m2s(0 to MAX_DM_CHANNELS-1), -- out   adb3_ocp_m2sT_array(0 to MAX_DM_CHANNELS-1)
      direct_masters_s2m => direct_masters_s2m(0 to MAX_DM_CHANNELS-1), -- in    adb3_ocp_s2mT_array(0 to MAX_DM_CHANNELS-1)
      -- MPTL Interface
      mptl_b2t           => mptl_b2t,                                   -- out   mptl_pins_t;
      mptl_t2b           => mptl_t2b,                                   -- in    mptl_pins_t;
      mptl_clk           => mptl_clk,                                   -- in    mptl_clk_t;
      -- MPTL Sideband Signals
      target_configured  => target_configured,                          -- in    std_logic
      target_gtp_online  => target_gtp_online,                          -- in    std_logic
      bridge_gtp_online  => bridge_gtp_online,                          -- out   std_logic
      -- MPTL Sideband Interface
      gpio_b2t           => gpio_b2t,                                   -- in    std_logic_vector(15 downto 0)
      gpio_t2b           => gpio_t2b,                                   -- out   std_logic_vector(15 downto 0)
      mptl_status        => mptl_status,                                -- out   std_logic_vector(63 downto 0))
      mptl_config        => mptl_config);                               -- in    std_logic_vector(31 downto 0))
  end generate;

  inst_syn_ngc_6t1_g : if (TARGET_USE = SIM_MPTL)and(BOARD_TYPE = ADM_XRC_6T1) generate
    -- full OCP-MPTL-OCP interface
    mptl128_interface_bridge_6t1_i : mptl128_interface_bridge_6t1
    port map(
      -- OCP Interface
      ocp_clk            => ocp_clk,                       -- in    std_logic
      direct_slave_m2s   => direct_slave_m2s,              -- in    adb3_ocp_m2sT
      direct_slave_s2m   => direct_slave_s2m,              -- out   adb3_ocp_s2mT
      dma_channels_m2s   => dma_channels_m2s(0 to 3),      -- in    adb3_ocp_m2sT_array(0 to 3)
      dma_channels_s2m   => dma_channels_s2m(0 to 3),      -- out   adb3_ocp_s2mT_array(0 to 3)
      direct_masters_m2s => direct_masters_m2s(0 to 2),    -- out   adb3_ocp_m2sT
      direct_masters_s2m => direct_masters_s2m(0 to 2),    -- in    adb3_ocp_s2mT
      -- MPTL Interface
      mptl_tx            => mptl_b2t,                      -- out   mptl_pins_type;
      mptl_rx            => mptl_t2b,                      -- in    mptl_pins_type;
      mptl_clk           => mptl_clk,                      -- in    mptl_clk_type;
      -- MPTL Sideband Signals
      target_configured  => target_configured,             -- in    std_logic
      target_gtp_online  => target_gtp_online,             -- in    std_logic
      bridge_gtp_online  => bridge_gtp_online,             -- out   std_logic
      -- MPTL Sideband Interface
      gpio_in            => gpio_b2t,                      -- in    std_logic_vector(15 downto 0)
      gpio_out           => gpio_t2b,                      -- out   std_logic_vector(15 downto 0)
      mptl_status        => mptl_status,                   -- out   std_logic_vector(63 downto 0)
      mptl_config        => mptl_config);                  -- in    std_logic_vector(31 downto 0))
  end generate;

  inst_syn_ngc_6tl_g : if (TARGET_USE = SIM_MPTL)and(BOARD_TYPE = ADM_XRC_6TL) generate
    -- full OCP-MPTL-OCP interface
    mptl64par_interface_bridge_6tl_i : mptl64par_interface_bridge_6tl
    port map(
      -- OCP Interface
      ocp_clk            => ocp_clk,                       -- in    std_logic
      direct_slave_m2s   => direct_slave_m2s,              -- in    adb3_ocp_m2sT
      direct_slave_s2m   => direct_slave_s2m,              -- out   adb3_ocp_s2mT
      dma_channels_m2s   => dma_channels_m2s(0 to 1),      -- in    adb3_ocp_m2sT_array(0 to 1)
      dma_channels_s2m   => dma_channels_s2m(0 to 1),      -- out   adb3_ocp_s2mT_array(0 to 1)
      direct_master_m2s  => direct_masters_m2s(0),         -- out   adb3_ocp_m2sT
      direct_master_s2m  => direct_masters_s2m(0),         -- in    adb3_ocp_s2mT
      -- MPTL Interface
      mptl_tx            => mptl_b2t,                      -- out   mptl_pins_type;
      mptl_rx            => mptl_t2b,                      -- in    mptl_pins_type;
      mptl_clk           => mptl_clk,                      -- in    mptl_clk_type;
      -- MPTL Sideband Signals
      target_configured  => target_configured,             -- in    std_logic
      target_gtp_online  => target_gtp_online,             -- in    std_logic
      bridge_gtp_online  => bridge_gtp_online,             -- out   std_logic
      -- MPTL Sideband Interface
      gpio_b2t           => gpio_b2t,                      -- in    std_logic_vector(15 downto 0)
      gpio_t2b           => gpio_t2b,                      -- out   std_logic_vector(15 downto 0)
      mptl_status        => mptl_status,                   -- out   std_logic_vector(63 downto 0)
      debug              => open);                         -- out   std_logic_vector(255 downto 0))
  end generate;

  -- Connect MPTL sideband ouputs
  mptl_bridge_gtp_online_n <= not(bridge_gtp_online);

end;